@@ -56,6 +56,20 @@ GlobusDebugDefine(GLOBUS_USAGE);
     }
 #endif
 
+#define GlobusUsageTr(_msg, _len, _from, _to)     \
+do                                                \
+{                                                 \
+    char *                  _ptr;                 \
+    char *                  _stop;                \
+    _stop = (char *) _msg + _len;                 \
+    _ptr = strchr((char * ) _msg, _from);         \
+    while(_ptr != NULL && _ptr <  _stop)          \
+    {                                             \
+        *_ptr = _to;                              \
+        _ptr = strchr((char * ) _ptr, _from);     \
+    }                                             \
+} while(0)
+
 #define PACKET_SIZE 1472
 
 static int
@@ -82,8 +96,8 @@ typedef struct globus_usage_stats_handle_s
     globus_xio_handle_t                 xio_handle;
     globus_list_t *                     xio_desc_list;
     const char *                        optout;
-    int                                 header_length;
-    int                                 data_length;
+    size_t                              header_length;
+    size_t                              data_length;
     unsigned char                       data[PACKET_SIZE];
 } globus_i_usage_stats_handle_t;
 
@@ -207,6 +221,7 @@ globus_usage_stats_handle_init(
     int                                 count;
     char                                hostname[255];
     int                                 rc = 0;
+    int                                 i;
 
     new_handle = globus_calloc(1, sizeof(globus_i_usage_stats_handle_t));
     if(!new_handle)
@@ -232,7 +247,7 @@ globus_usage_stats_handle_init(
     new_handle->code = htons(code);
     new_handle->version = htons(version);
 
-    memset(new_handle->data, 0, 1472);
+    memset(new_handle->data, 0, PACKET_SIZE);
 
     memcpy(new_handle->data + new_handle->data_length, 
            (void *)&new_handle->code, 2);
@@ -278,16 +293,19 @@ globus_usage_stats_handle_init(
         new_handle->data_length += 12;
     }
 
-    memcpy(new_handle->data + new_handle->data_length, host, count);
-    new_handle->data_length += count;
+    for (i = 0; i < count; i++)
+    {
+        new_handle->data[new_handle->data_length++] = (unsigned char) host[i];
+    }
 
     /* timestamp will go here */
     new_handle->data_length += 4;
 
     if(globus_libc_gethostname(hostname, 255) == 0)
     {
-        new_handle->data_length += sprintf(new_handle->data + new_handle->data_length,
-                               "HOSTNAME=%s", hostname);
+        new_handle->data_length += 
+            sprintf((char *) new_handle->data + new_handle->data_length,
+                "HOSTNAME=%s", hostname);
     }
     new_handle->header_length = new_handle->data_length;
     
@@ -421,7 +439,6 @@ globus_l_usage_stats_write_packet(
     nstamp = htonl(stamp.tv_sec);
     memcpy(handle->data + GLOBUS_L_USAGE_STATS_TIMESTAMP_OFFSET, 
            (void *)&nstamp, 4);
-
     targets_list = handle->xio_desc_list;
     server_list = handle->targets;
     while(targets_list)
@@ -503,12 +520,14 @@ globus_usage_stats_send_array(
         {
             const char *                key;
             const char *                value;
-            int                         length; 
-
+            size_t                      length; 
+            size_t                      val_len;
+            const unsigned char *       val_start;
             key = key_array[i];
             value = value_array[i];
             
-            length = strlen(key) + strlen(value);
+            val_len = strlen(value);
+            length = strlen(key) + val_len;
 
             if(index(value, ' '))
             {
@@ -525,8 +544,11 @@ globus_usage_stats_send_array(
                             "Parameters don't fit into one packet"));
                 }
                 handle->data_length += sprintf(
-                    handle->data + handle->data_length,
+                    (char *) handle->data + handle->data_length,
                     "%s=\"%s\" ", key, value);
+                val_start = handle->data + handle->data_length - val_len - 2;
+                GlobusUsageTr(val_start, val_len, '"', '\'');
+                GlobusUsageTr(val_start, val_len, '\n', ' ');
             }
             else
             {
@@ -543,8 +565,11 @@ globus_usage_stats_send_array(
                             "Parameters don't fit into one packet"));
                 }
                 handle->data_length += sprintf(
-                    handle->data + handle->data_length,
+                    (char *) handle->data + handle->data_length,
                     "%s=%s ", key, value);
+                val_start = handle->data + handle->data_length - val_len - 1;
+                GlobusUsageTr(val_start, val_len, '"', '\'');
+                GlobusUsageTr(val_start, val_len, '\n', ' ');
             }
         }
     }
@@ -619,8 +644,12 @@ globus_usage_stats_vsend(
         {
             const char *                key = va_arg(ap, char *);
             const char *                value = va_arg(ap, char *);
-            int                         length = strlen(key) +
-                                                 strlen(value);
+            size_t                      length;
+            size_t                      val_len;
+            const unsigned char *       val_start;
+            
+            val_len = strlen(value);
+            length = strlen(key) + val_len;
 
             if(index(value, ' '))
             {
@@ -637,8 +666,11 @@ globus_usage_stats_vsend(
                             "Parameters don't fit into one packet"));
                 }
                 handle->data_length += sprintf(
-                    handle->data + handle->data_length,
+                    (char *) handle->data + handle->data_length,
                     "%s=\"%s\" ", key, value);
+                val_start = handle->data + handle->data_length - val_len - 2;
+                GlobusUsageTr(val_start, val_len, '"', '\'');
+                GlobusUsageTr(val_start, val_len, '\n', ' ');
             }
             else
             {
@@ -655,8 +687,11 @@ globus_usage_stats_vsend(
                             "Parameters don't fit into one packet"));
                 }
                 handle->data_length += sprintf(
-                    handle->data + handle->data_length,
+                    (char *) handle->data + handle->data_length,
                     "%s=%s ", key, value);
+                val_start = handle->data + handle->data_length - val_len - 1;
+                GlobusUsageTr(val_start, val_len, '"', '\'');
+                GlobusUsageTr(val_start, val_len, '\n', ' ');
             }
         }
     }
